"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.expectProperty = exports.typecheckSchemaAt = void 0;
const guards_1 = require("./guards");
const diagnostics_1 = require("./diagnostics");
function typecheckSchemaAt(schema, path) {
    if (guards_1.isArray(schema)) {
        return typecheckImplicitObjectSchemaAt(schema, path);
    }
    else if (guards_1.isRecord(schema)) {
        if (guards_1.hasTypeProperty(schema, "object")) {
            return typecheckExplicitObjectSchemaAt(schema, path);
        }
        else if (guards_1.hasProperty(schema, "length")) {
            return typecheckArraySchemaAt(schema, path);
        }
        else if (guards_1.hasProperty(schema, "type")) {
            return typecheckExplicitNumberOrExplicitStringSchemaAt(schema, path);
        }
        else {
            throw diagnostics_1.error({
                expected: [expectProperty("type"), expectProperty("length")],
                actual: schema,
                path,
            });
        }
    }
    else if (schema === "number" || schema === "string") {
        return schema;
    }
    else {
        throw diagnostics_1.error({
            expected: ["object", "array", '"number"', '"string"'],
            actual: schema,
            path,
        });
    }
}
exports.typecheckSchemaAt = typecheckSchemaAt;
function typecheckExplicitNumberOrExplicitStringSchemaAt(schema, path) {
    const { type } = schema;
    // NOTE: cannot use || for the next two conditions because otherwise
    // TypeScript 3.8 assumes ArraySchema
    if (type === "string") {
        return { type };
    }
    else if (type === "number") {
        return { type };
    }
    else {
        throw diagnostics_1.error({
            expected: ['"string"', '"number"'],
            actual: schema.type,
            path: diagnostics_1.push(path, "type"),
        });
    }
}
function typecheckArraySchemaAt(schema, path) {
    const { length } = schema;
    if (typeof length !== "string") {
        throw diagnostics_1.error({
            expected: ["string"],
            actual: length,
            path: diagnostics_1.push(path, "length"),
        });
    }
    const result = { length };
    if (guards_1.hasProperty(schema, "type")) {
        if (schema.type === "number" ||
            schema.type === "string" ||
            schema.type === "array") {
            result.type = schema.type;
        }
        else {
            throw diagnostics_1.error({
                expected: ['"number"', '"string"', '"array"'],
                actual: schema.type,
                path: diagnostics_1.push(path, "type"),
            });
        }
    }
    if (guards_1.hasProperty(schema, "items")) {
        result.items = typecheckSchemaAt(schema.items, diagnostics_1.push(path, "items"));
    }
    return result;
}
function typecheckImplicitObjectSchemaAt(schema, path) {
    return typecheckPropertySchemaArray(schema, path);
}
function typecheckPropertySchemaArray(schema, path) {
    return schema.map((property, index) => typecheckPropertySchemaAt(property, diagnostics_1.push(path, index)));
}
function typecheckExplicitObjectSchemaAt(schema, path) {
    if (!guards_1.hasProperty(schema, "properties")) {
        throw diagnostics_1.error({
            expected: [expectProperty("properties")],
            actual: schema,
            path,
        });
    }
    const { properties } = schema;
    if (!guards_1.isArray(properties)) {
        throw diagnostics_1.error({
            expected: ["array"],
            actual: properties,
            path: diagnostics_1.push(path, "properties"),
        });
    }
    else {
        return {
            type: schema.type,
            properties: typecheckPropertySchemaArray(properties, diagnostics_1.push(path, "properties")),
        };
    }
}
function typecheckPropertySchemaAt(schema, path) {
    if (guards_1.isRecord(schema)) {
        return typecheckExplicitPropertySchemaAt(schema, path);
    }
    else if (guards_1.isArray(schema)) {
        return typecheckArrayPropertySchemaAt(schema, path);
    }
    else if (typeof schema === "string") {
        return schema;
    }
    else {
        throw diagnostics_1.error({
            expected: ["object", "array", "string"],
            actual: schema,
            path,
        });
    }
}
function typecheckExplicitPropertySchemaAt(schema, path) {
    if (!guards_1.hasProperty(schema, "name")) {
        throw diagnostics_1.error({ expected: [expectProperty("name")], actual: schema, path });
    }
    if (!guards_1.hasProperty(schema, "value")) {
        throw diagnostics_1.error({ expected: [expectProperty("value")], actual: schema, path });
    }
    const { name, value } = schema;
    if (typeof name !== "string") {
        throw diagnostics_1.error({
            expected: ["string"],
            actual: name,
            path: diagnostics_1.push(path, "name"),
        });
    }
    return {
        name,
        value: typecheckSchemaAt(value, diagnostics_1.push(path, "value")),
    };
}
function typecheckArrayPropertySchemaAt(schema, path) {
    if (schema.length < 2 || schema.length > 3) {
        throw diagnostics_1.error({
            expected: [2, 3],
            actual: schema.length,
            path: diagnostics_1.push(path, "length"),
        });
    }
    const [name, length] = schema;
    if (typeof name !== "string") {
        throw diagnostics_1.error({
            expected: ["string"],
            actual: name,
            path: diagnostics_1.push(path, 0),
        });
    }
    if (typeof length !== "string") {
        throw diagnostics_1.error({
            expected: ["string"],
            actual: length,
            path: diagnostics_1.push(path, 1),
        });
    }
    if (schema.length === 2) {
        return [name, length];
    }
    const itemSchema = typecheckSchemaAt(schema[2], diagnostics_1.push(path, 2));
    return [name, length, itemSchema];
}
function expectProperty(property) {
    return `a '${property}' property`;
}
exports.expectProperty = expectProperty;
//# sourceMappingURL=typecheckAt.js.map