"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tap_1 = require("tap");
const typecheckAt_1 = require("./typecheckAt");
tap_1.test("valid schema with all combinations of valid schemas", t => {
    const schemaIn = {
        // explicit object schema
        type: "object",
        properties: [
            // explicit property schema
            {
                name: "property1",
                value: "number"
            },
            // number property
            "property2",
            // array of number property
            ["property3", "length3"],
            // explicit array property
            ["property4", "length4", "number"],
            // implicit object schema, empty
            { name: "property5", value: [] },
            // implicit object schema, not empty
            { name: "property6", value: ["subproperty1", "subproperty2"] },
            // implicit array of number schema
            { name: "property7", value: { length: "length7" } },
            // array of number schema
            { name: "property8", value: { length: "length8", type: "number" } },
            // array of string schema
            { name: "property9", value: { length: "length8", type: "string" } },
            // array of number schema
            { name: "property11", value: { length: "length8", type: "array" } },
            // explicit array schema
            {
                name: "property12",
                value: { length: "length8", type: "array", items: "number" }
            },
            // array of number schema
            { name: "property13", value: { length: "length8", items: "number" } },
            // explicit number
            { name: "property14", value: { type: "number" } },
            // explicit string
            { name: "property15", value: { type: "string" } }
        ]
    };
    const schemaOut = typecheckAt_1.typecheckSchemaAt(schemaIn, []);
    t.deepEqual(schemaOut, schemaIn);
    t.end();
});
tap_1.test("explicit property schema, missing name", t => {
    const schema = [{}];
    t.throws(() => {
        typecheckAt_1.typecheckSchemaAt(schema, []);
    }, { expected: [typecheckAt_1.expectProperty("name")], actual: schema[0], path: [0] });
    t.end();
});
tap_1.test("explicit property schema, missing value", t => {
    const schema = [{ name: "name" }];
    t.throws(() => {
        typecheckAt_1.typecheckSchemaAt(schema, []);
    }, { expected: [typecheckAt_1.expectProperty("value")], actual: schema[0], path: [0] });
    t.end();
});
tap_1.test("explicit property schema, mistyped name", t => {
    const schema = [{ name: 1, value: "value" }];
    t.throws(() => {
        typecheckAt_1.typecheckSchemaAt(schema, []);
    }, { expected: ["string"], actual: schema[0].name, path: [0, "name"] });
    t.end();
});
tap_1.test("explicit property schema, mistyped value", t => {
    const schema = [{ name: "name", value: null }];
    t.throws(() => {
        typecheckAt_1.typecheckSchemaAt(schema, []);
    }, { actual: schema[0].value, path: [0, "value"] });
    t.end();
});
tap_1.test("array property schema, not enough elements", t => {
    const schema = [[]];
    t.throws(() => {
        typecheckAt_1.typecheckSchemaAt(schema, []);
    }, { expected: [2, 3], actual: schema[0].length, path: [0, "length"] });
    t.end();
});
tap_1.test("array property schema, too much elements", t => {
    const schema = [[1, 2, 3, 4]];
    t.throws(() => {
        typecheckAt_1.typecheckSchemaAt(schema, []);
    }, { expected: [2, 3], actual: schema[0].length, path: [0, "length"] });
    t.end();
});
tap_1.test("array property schema, mistyped name", t => {
    const schema = [[1, "length"]];
    t.throws(() => {
        typecheckAt_1.typecheckSchemaAt(schema, []);
    }, { expected: ["string"], actual: schema[0][0], path: [0, 0] });
    t.end();
});
tap_1.test("array property schema, mistyped length", t => {
    const schema = [["name", 1]];
    t.throws(() => {
        typecheckAt_1.typecheckSchemaAt(schema, []);
    }, { expected: ["string"], actual: schema[0][1], path: [0, 1] });
    t.end();
});
tap_1.test("array property schema, mistyped item schema", t => {
    const schema = [["name", "length", null]];
    t.throws(() => {
        typecheckAt_1.typecheckSchemaAt(schema, []);
    }, { actua: schema[0][2], path: [0, 2] });
    t.end();
});
tap_1.test("implicit object schema, mistyped property schema", t => {
    const schema = [null];
    t.throws(() => {
        typecheckAt_1.typecheckSchemaAt(schema, []);
    }, { expected: ["object", "array", "string"], actual: schema[0], path: [0] });
    t.end();
});
tap_1.test("explicit object schema, missing properties", t => {
    const schema = { type: "object" };
    t.throws(() => {
        typecheckAt_1.typecheckSchemaAt(schema, []);
    }, { expected: [typecheckAt_1.expectProperty("properties")], actual: schema, path: [] });
    t.end();
});
tap_1.test("explicit object schema, mistyped properties", t => {
    const schema = { type: "object", properties: null };
    t.throws(() => {
        typecheckAt_1.typecheckSchemaAt(schema, []);
    }, { expected: ["array"], actual: schema.properties, path: ["properties"] });
    t.end();
});
tap_1.test("explicit object schema, mistyped property schema", t => {
    const schema = { type: "object", properties: [null] };
    t.throws(() => {
        typecheckAt_1.typecheckSchemaAt(schema, []);
    }, {
        expected: ["object", "array", "string"],
        actual: schema.properties[0],
        path: ["properties", 0]
    });
    t.end();
});
tap_1.test("array schema, mistyped length", t => {
    const schema = { length: 1 };
    t.throws(() => {
        typecheckAt_1.typecheckSchemaAt(schema, []);
    }, { expected: ["string"], actual: schema.length, path: ["length"] });
    t.end();
});
tap_1.test("array schema, misvalued type", t => {
    const schema = { length: "length", type: "not a type" };
    t.throws(() => {
        typecheckAt_1.typecheckSchemaAt(schema, []);
    }, {
        expected: ['"number"', '"string"', '"array"'],
        actual: schema.type,
        path: ["type"]
    });
    t.end();
});
tap_1.test("array schema, mistyped item schema", t => {
    const schema = { length: "length", type: "string", items: null };
    t.throws(() => {
        typecheckAt_1.typecheckSchemaAt(schema, []);
    }, {
        expected: ["object", "array", '"number"', '"string"'],
        actual: schema.items,
        path: ["items"]
    });
    t.end();
});
tap_1.test("unknown record-like schema, misvalued type", t => {
    const schema = { type: "not a type" };
    t.throws(() => {
        typecheckAt_1.typecheckSchemaAt(schema, []);
    }, {
        expected: ['"string"', '"number"'],
        actual: schema.type,
        path: ["type"]
    });
    t.end();
});
tap_1.test("unknown record-like schema, no type and no length", t => {
    const schema = {};
    t.throws(() => {
        typecheckAt_1.typecheckSchemaAt(schema, []);
    }, {
        expected: [typecheckAt_1.expectProperty("type"), typecheckAt_1.expectProperty("length")],
        actual: schema,
        path: []
    });
    t.end();
});
tap_1.test("unknown schema", t => {
    const schema = null;
    t.throws(() => {
        typecheckAt_1.typecheckSchemaAt(schema, []);
    }, {
        expected: ["object", "array", '"number"', '"string"'],
        actual: schema,
        path: []
    });
    t.end();
});
//# sourceMappingURL=typecheckAt.test.js.map